@@ -137,7 +137,7 @@ class AWSProvider(Provider):
 
         ids = [srv.id for srv in aws_res]
         if len(ids) != 1:  # ids must be len of 1 as we provision one vm at the time
-            raise ProvisioningError("Unexpected number of instances provisioned.")
+            raise ProvisioningError("Unexpected number of instances provisioned.", req)
         # creating name for instance (visible in aws ec2 WebUI)
         taglist = [{"Key": "name", "Value": specs.get("name")}]
         for key in self.instance_tags:
