@@ -48,6 +48,8 @@ from mrack.providers.podman import PROVISIONER_KEY as PODMAN
 from mrack.providers.podman import PodmanProvider
 from mrack.providers.static import PROVISIONER_KEY as STATIC
 from mrack.providers.static import StaticProvider
+from mrack.providers.virt import PROVISIONER_KEY as VIRT
+from mrack.providers.virt import VirtProvider
 from mrack.utils import NoSuchFileHandler, load_yaml
 from mrack.version import VERSION
 
@@ -72,6 +74,7 @@ def init_providers():
     providers.register(BEAKER, BeakerProvider)
     providers.register(PODMAN, PodmanProvider)
     providers.register(STATIC, StaticProvider)
+    providers.register(VIRT, VirtProvider)
 
 
 def init_db(path):
